﻿using System;
using System.Collections.Generic;
using System.Linq;
using GenericLibrary;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace GenericLibraryTests
{
    [TestClass]
    public class CatalogListTests
    {

        private readonly Book _book1 = new Book(1, "Anders", "C#");
        private readonly Book _book2 = new Book(2, "John Sharp", "C# 2012");
        private CatalogList<int, Book> _books;

        [TestInitialize]
        public void Intialize()
        {
            _books = new CatalogList<int, Book> { _book1, _book2 };
        }

        [TestMethod]
        public void AddCountTest()
        {
            Assert.AreEqual(2, _books.Count);
            Book anotherBook = new Book(3, "Bard de Smet", "C# 5.0 Unleashed");
            bool added = _books.Add(anotherBook);
            Assert.IsTrue(added);
            Assert.AreEqual(3, _books.Count);
        }

        [TestMethod]
        public void FindTest()
        {
            Book b = _books.Find(1);
            Assert.AreEqual(b, _book1);
            Assert.IsNull(_books.Find(100));
        }

        [TestMethod]
        public void GetAllTest()
        {
            IEnumerable<Book> allBooks = _books.GetAll();
            Assert.AreEqual(2, allBooks.Count());

            IEnumerable<Book> someBooks = _books.GetAll(book => book.Author.StartsWith("A"));
            Assert.AreEqual(1, someBooks.Count());
        }

        [TestMethod]
        public void GetAll2Test()
        {
            IReadOnlyCollection<Book> allBooks = _books.GetAll2();
            Assert.AreEqual(2, allBooks.Count);

            //IEnumerable<Book> someBooks = _books.GetAll2(book => book.Author.StartsWith("A"));
            //Assert.AreEqual(1, someBooks.Count());
        }

        [TestMethod]
        public void GetAll3Test()
        {
            ICollection<Book> bo = _books.GetAll3();
            try
            {
                bo.Add(null);
            }
            catch (NotSupportedException)
            {

            }
        }

        [TestMethod]
        public void GetAllPredicate()
        {
            IEnumerable<Book> bo = _books.GetAll(book => book.Author.StartsWith("A"));
            Assert.AreEqual(1, bo.Count());
        }

        [TestMethod]
        public void GetAllCondition()
        {
            ICondition<Book> condition = new BookAuthorCondition("A");
            IEnumerable<Book> bo = _books.GetAll(condition);
            Assert.AreEqual(1, bo.Count());
        }
    }

    class BookAuthorCondition : ICondition<Book>
    {
        public BookAuthorCondition(string authorNameStart)
        {
            AuthorNameStart = authorNameStart;
        }

        public String AuthorNameStart { get; private set; }

        public bool Evaluate(Book element)
        {
            return element.Author.StartsWith(AuthorNameStart);
        }
    }

    class Book : IIdentifiable<int>, IEquatable<Book>
    {
        public String Author { get; set; }
        public String Title { get; set; }

        private readonly int _id;

        public int Id { get { return _id; } }

        public Book(int id, string author, string title)
        {
            _id = id;
            Author = author;
            Title = title;
        }

        public override string ToString()
        {
            return string.Format("{0} {1} {2}", Id, Author, Title);
        }

        public bool Equals(Book other)
        {
            if (ReferenceEquals(null, other)) return false;
            if (ReferenceEquals(this, other)) return true;
            return _id == other._id;
        }

        public override bool Equals(object obj)
        {
            if (ReferenceEquals(null, obj)) return false;
            if (ReferenceEquals(this, obj)) return true;
            if (obj.GetType() != this.GetType()) return false;
            return Equals((Book)obj);
        }

        public override int GetHashCode()
        {
            return _id;
        }
    }
}